# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Contribution.card_expiration_date'
        db.add_column('leetchi_contribution', 'card_expiration_date',
                      self.gf('django.db.models.fields.DateField')(null=True),
                      keep_default=False)

        # Adding field 'Contribution.card_number'
        db.add_column('leetchi_contribution', 'card_number',
                      self.gf('django.db.models.fields.CharField')(max_length=100, null=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Contribution.card_expiration_date'
        db.delete_column('leetchi_contribution', 'card_expiration_date')

        # Deleting field 'Contribution.card_number'
        db.delete_column('leetchi_contribution', 'card_number')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('sluggable.fields.SluggableField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'djleetchi.beneficiary': {
            'Meta': {'object_name': 'Beneficiary', 'db_table': "'leetchi_beneficiary'"},
            'bank_account_bic': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'bank_account_iban': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'bank_account_owner_address': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'bank_account_owner_name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'beneficiary': ('djleetchi.fields.ResourceField', [], {'null': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'djleetchi.contribution': {
            'Meta': {'object_name': 'Contribution', 'db_table': "'leetchi_contribution'"},
            'amount': ('django.db.models.fields.IntegerField', [], {}),
            'card_expiration_date': ('django.db.models.fields.DateField', [], {'null': 'True'}),
            'card_number': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'client_fee_amount': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'contribution': ('djleetchi.fields.ResourceField', [], {'null': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_completed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_success': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'return_url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'template_url': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.PositiveSmallIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"}),
            'wallet': ('djleetchi.fields.ResourceField', [], {})
        },
        u'djleetchi.refund': {
            'Meta': {'object_name': 'Refund', 'db_table': "'leetchi_refund'"},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'contribution': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['djleetchi.Contribution']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_completed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_success': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'refund': ('djleetchi.fields.ResourceField', [], {'null': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'djleetchi.strongauthentication': {
            'Meta': {'object_name': 'StrongAuthentication', 'db_table': "'leetchi_strongauthentication'"},
            'beneficiary': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'strong_authentication'", 'null': 'True', 'to': u"orm['djleetchi.Beneficiary']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_completed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_succeeded': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'strong_authentication': ('djleetchi.fields.ResourceField', [], {'null': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'djleetchi.transfer': {
            'Meta': {'object_name': 'Transfer', 'db_table': "'leetchi_transfer'"},
            'amount': ('django.db.models.fields.IntegerField', [], {}),
            'beneficiary': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'beneficiaries'", 'to': u"orm['auth.User']"}),
            'beneficiary_wallet': ('djleetchi.fields.ResourceField', [], {}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'payer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'payers'", 'to': u"orm['auth.User']"}),
            'transfer': ('djleetchi.fields.ResourceField', [], {'null': 'True'})
        },
        u'djleetchi.transferrefund': {
            'Meta': {'object_name': 'TransferRefund', 'db_table': "'leetchi_transferrefund'"},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'transfer': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['djleetchi.Transfer']"}),
            'transfer_refund': ('djleetchi.fields.ResourceField', [], {'null': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'djleetchi.wallet': {
            'Meta': {'unique_together': "(('wallet', 'content_type', 'object_id'),)", 'object_name': 'Wallet', 'db_table': "'leetchi_wallet'"},
            'amount': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_synced': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('djleetchi.fields.ResourceField', [], {'null': 'True', 'blank': 'True'}),
            'wallet': ('djleetchi.fields.ResourceField', [], {'null': 'True', 'blank': 'True'})
        },
        u'djleetchi.withdrawal': {
            'Meta': {'object_name': 'Withdrawal', 'db_table': "'leetchi_withdrawal'"},
            'amount': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'beneficiary': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['djleetchi.Beneficiary']", 'null': 'True', 'blank': 'True'}),
            'client_fee_amount': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_completed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_succeeded': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'}),
            'wallet': ('djleetchi.fields.ResourceField', [], {'null': 'True', 'blank': 'True'}),
            'withdrawal': ('djleetchi.fields.ResourceField', [], {'null': 'True'})
        }
    }

    complete_apps = ['djleetchi']


import requests

from celery.task import task

from leetchi.exceptions import APIError, DecodeError

from django.core.files.storage import default_storage as storage


@task
def create_strong_authentication(user_id, beneficiary_id, filepaths=None):
    from .compat import User
    from .models import StrongAuthentication, Beneficiary
    from .api import handler

    logger = create_strong_authentication.get_logger()

    user = User.objects.get(pk=user_id)

    beneficiary = Beneficiary.objects.get(pk=beneficiary_id)

    auth = StrongAuthentication(user=user, beneficiary=beneficiary)

    try:
        auth.save()

        if filepaths:
            strong_authentication = auth.strong_authentication

            for filepath in filepaths:
                result = requests.post(strong_authentication.url_request, files={
                    'StrongValidationDto.Picture': storage.open(filepath).file
                })

                logger.info(u'Uploading file for <User: %s> and <StrongAuthentication %s>: %s' % (user_id,
                                                                                                  auth.pk,
                                                                                                  result.status_code))

            strong_authentication.is_transmitted = True
            strong_authentication.save(handler)
    except APIError, exc:
        logger.exception(exc)


@task
def sync_status(resource_klass, resource_id):
    from leetchi.base import DoesNotExist

    logger = sync_status.get_logger()

    try:

        resource = resource_klass.objects.get(pk=resource_id)
        resource.sync_status()

        resource_name = resource_klass.__name__

        if resource.is_completed:
            if resource.is_success:
                logger.info(u'[%s]: set succeeded for %d, from leetchi resource %d' %
                            (resource_name, resource.pk, resource.resource_id))
            else:
                logger.info(u'[%s]: set aborted for %d, from leetchi resource %d' %
                            (resource_name, resource.pk, resource.resource_id))
        else:
            logger.info(u'[%s]: do nothing for %d, from leetchi resource %d' %
                        (resource_name, resource.pk, resource.resource_id))

    except (APIError, DecodeError, DoesNotExist), e:
        logger.exception(e)


@task
def sync_resource(resource_klass, resource_id):
    from leetchi.base import DoesNotExist

    logger = sync_resource.get_logger()

    try:

        resource = resource_klass.objects.get(pk=resource_id)
        resource.sync()

        resource_name = resource_klass.__name__

        logger.info(u'[%s]: syncing %d, from leetchi resource %d' %
                    (resource_name, resource.pk, resource.resource_id))
    except (APIError, DecodeError, DoesNotExist), e:
        logger.exception(e)


@task
def update_user(user_id, **kwargs):
    from .compat import User
    from .api import handler
    from .models import Wallet

    user = User.objects.get(pk=user_id)

    wallet = Wallet.objects.get_for_model(user)

    if wallet:
        payer = wallet.user

        for k, v in kwargs.items():
            setattr(payer, k, v)

        payer.save(handler)


@task
def sync_amount(wallet_id):
    from .models import Wallet

    logger = sync_amount.get_logger()

    wallet = Wallet.objects.get(pk=wallet_id)
    wallet.sync_amount()

    logger.info('[Wallet] Syncing amount for %s' % wallet_id)



from django.utils.functional import memoize

from leetchi.api import LeetchiAPI

from . import settings


def _get_handler():
    return LeetchiAPI(settings.API_PARTNER_ID,
                      settings.API_PRIVATE_KEY,
                      settings.API_PRIVATE_KEY_PASSWORD,
                      sandbox=settings.API_USE_SANDBOX,
                      host=settings.API_HOST)


handler = memoize(_get_handler, {}, 0)()



import re

from django import template
from django.contrib.contenttypes.models import ContentType

from djleetchi.models import (Contribution, Refund, Wallet,
                              Transfer, TransferRefund, Withdrawal)

register = template.Library()


class ResourcesNode(template.Node):
    def __init__(self, obj, var_name, klass):
        self.obj = template.Variable(obj)
        self.var_name = var_name
        self.klass = klass

    def render(self, context):
        try:
            obj = self.obj.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        results = self.klass.objects.filter(content_type=ContentType.objects.get_for_model(obj),
                                            object_id=obj.pk)

        if self.var_name:
            context[self.var_name] = results

        return ''


@register.tag
def get_resources_for(parser, token, klass):
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError, "%s tag requires arguments" % token.contents.split()[0]
    m = re.search(r'(.*?) as (\w+)', arg)

    if not m:
        raise template.TemplateSyntaxError, "%s tag had invalid arguments" % tag_name
    user, var_name = m.groups()

    return ResourcesNode(user, var_name, klass)


@register.tag
def get_contributions_for(parser, token):
    return get_resources_for(parser, token, Contribution)


@register.tag
def get_refunds_for(parser, token):
    return get_resources_for(parser, token, Refund)


@register.tag
def get_withdrawals_for(parser, token):
    return get_resources_for(parser, token, Withdrawal)


@register.tag
def get_transfers_for(parser, token):
    return get_resources_for(parser, token, Transfer)


@register.tag
def get_transferrefunds_for(parser, token):
    return get_resources_for(parser, token, TransferRefund)


@register.tag
def get_wallets_for(parser, token):
    return get_resources_for(parser, token, Wallet)



# -*- coding: utf-8 -*-
from django.http import HttpResponse

try:
    import simplejson as json
except ImportError:
    import json # NOQA


class JSONResponse(HttpResponse):

    """Represent a HTTP response with a Content-Type of 'application/json'.

    This subclass of `django.http.HttpResponse` will remove most of the
    necessary boilerplate in returning JSON responses from views. The object
    to be serialized is passed in as the only positional argument, and the
    other keyword arguments are as usual for `HttpResponse`, with one
    exception: an optional 'callback' argument may be given which will wrap
    the response with a JavaScript function call to the given function name.
    """

    def __init__(self, obj, indent=0, cls=None, **kwargs):
        params = {
            'indent': indent
        }

        if cls:
            params['cls'] = cls

        kwargs['content'] = json.dumps(obj, **params)

        if 'callback' in kwargs:
            callback = kwargs.pop('callback')
            if callback:
                kwargs['content'] = u'%s(%s);' % (callback, kwargs['content'])
                kwargs.setdefault('content_type', 'application/javascript')

        kwargs.setdefault('content_type', 'application/json')
        super(JSONResponse, self).__init__(**kwargs)



import logging

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site

from leetchi.exceptions import APIError, DecodeError

from .models import Contribution, Transfer, Beneficiary, Withdrawal, get_pending_amount
from .helpers import get_payer

from django_iban.forms import IBANFormField, SWIFTBICFormField

logger = logging.getLogger('django.leetchi')


class WithdrawalForm(forms.ModelForm):
    bank_account_owner_name = forms.CharField(label=_(u'Name of the bank account owner'),
                                              required=True)
    bank_account_owner_address = forms.CharField(label=_(u'Address of the bank account owner'),
                                                 required=True)
    bank_account_iban = IBANFormField(label=_(u'IBAN of the bank account owner'),
                                      required=True,
                                      help_text=_(u'Without spaces'))
    bank_account_bic = SWIFTBICFormField(label=_(u'BIC of the bank account owner'),
                                         required=True,
                                         help_text=_(u'Without spaces'))

    class Meta:
        model = Withdrawal
        fields = ('amount', 'client_fee_amount', )

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        self.wallet = kwargs.pop('wallet', None)
        self.content_object = kwargs.pop('content_object', None)
        self.sync = kwargs.pop('sync', True)

        super(WithdrawalForm, self).__init__(*args, **kwargs)

        self.fields['amount'] = forms.DecimalField(label=_('Amount'), help_text=_('Value in decimal, e.g. 20.50'))

    def is_personal_amount_enough(self, amount):
        payer = get_payer(self.user)

        personal_amount = payer.personal_wallet_amount_converted

        return personal_amount >= float(amount)

    def clean_amount(self):
        amount = self.cleaned_data.get('amount')

        if self.content_object or not self.user:
            return amount

        payer = get_payer(self.user)

        amount_converted = payer.personal_wallet_amount_converted

        if not self.is_personal_amount_enough(amount):
            error_message = _('Your personal amount %(personal_amount)s is '
                              'not enough to make this withdrawal %(amount)s')

            raise forms.ValidationError(error_message % {
                'personal_amount': amount_converted,
                'amount': amount
            })

        withdrawal_amount = get_pending_amount(self.user) / 100

        if withdrawal_amount:
            if not self.is_personal_amount_enough(amount + withdrawal_amount):
                error_message = _('You can\'t make a withdrawal of %(amount)s, you already have '
                                  '%(withdrawal_amount)s of pending withdrawals and your personal amount is only %(personal_amount)s')

                raise forms.ValidationError(error_message % {
                    'personal_amount': amount_converted,
                    'amount': amount,
                    'withdrawal_amount': withdrawal_amount
                })

        return amount

    def save(self):
        try:
            beneficiary = Beneficiary(user=self.user)

            for k, v in self.cleaned_data.items():
                setattr(beneficiary, k, v)

            beneficiary.save(sync=True)

            self.instance.beneficiary = beneficiary

            if self.wallet:
                self.instance.wallet = self.wallet

            self.instance.user = self.user
            self.instance.amount = self.cleaned_data['amount'] * 100
            self.instance.content_object = self.content_object or self.user
            self.instance.save(sync=self.sync)
        except (APIError, DecodeError), e:
            logger.error(e)

            self.instance.error = e
        else:
            self.instance.error = None

        return self.instance


class ContributionForm(forms.Form):
    amount = forms.DecimalField(label=_("Amount"), widget=forms.TextInput,
                                help_text=_('Value in decimal, e.g. 20.50'))

    def save(self, request, user, return_url, template_url=None):
        try:
            current_site = Site.objects.get_current()

            protocol = getattr(settings, "DEFAULT_HTTP_PROTOCOL", "http")

            absolute_url = request.META.get('HTTP_HOST', current_site.domain)

            return_url = u"%s://%s%s" % (
                protocol,
                unicode(absolute_url),
                return_url
            )

            amount = int(float(self.cleaned_data.get('amount')) * 100)

            contribution = Contribution()
            contribution.content_object = user
            contribution.user = request.user
            contribution.wallet_id = 0
            contribution.amount = amount
            contribution.return_url = return_url
            contribution.target = user

            template_url = u"https://%s%s" % (
                unicode(absolute_url),
                template_url
            )

            contribution.template_url = template_url

            contribution.save(sync=True)
        except (APIError, DecodeError), e:
            logger.error(e)
            return False
        else:
            return contribution


class TransferForm(forms.Form):
    amount = forms.CharField(label=_("Amount"), widget=forms.TextInput,
                             help_text=_('Value in decimal, e.g. 20.50'))

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user')
        super(TransferForm, self).__init__(*args, **kwargs)

    def clean_amount(self):

        payer = get_payer(self.user)

        amount = self.cleaned_data.get('amount')

        if not self.is_personal_amount_enough(amount):
            error_message = _('Your personal amount %(personal_amount)s is \
                              not enough to make this transfer %(amount)s')

            amount_converted = payer.personal_wallet_amount_converted

            raise forms.ValidationError(error_message % {
                'personal_amount': amount_converted,
                'amount': amount
            })

        return amount

    def is_personal_amount_enough(self, amount):
        payer = get_payer(self.user)

        personal_amount = payer.personal_wallet_amount_converted

        return personal_amount >= float(amount)

    def save(self, payer_user, beneficiary_user):
        try:
            transfer = Transfer()
            transfer.content_object = beneficiary_user
            transfer.user = payer_user
            transfer.payer = payer_user
            transfer.amount = int(float(self.cleaned_data.get('amount')) * 100)
            transfer.beneficiary = beneficiary_user
            transfer.beneficiary_wallet_id = 0
            transfer.save()
        except (APIError, DecodeError), e:
            logger.error(e)
            return False
        else:
            return transfer



import six

from collections import defaultdict

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from .util import load_class


def queryset_to_dict(qs, key='pk', singular=True):
    """
    Given a queryset will transform it into a dictionary based on ``key``.
    """
    if singular:
        result = {}
        for u in qs:
            result.setdefault(getattr(u, key), u)
    else:
        result = defaultdict(list)
        for u in qs:
            result[getattr(u, key)].append(u)
    return result


def _get_payer(user):
    from .models import Wallet

    return Wallet.objects.get_for_model(user).user


get_payer = getattr(settings, 'LEETCHI_PAYER_HELPER', _get_payer)

if not get_payer:
    raise ImproperlyConfigured('The required setting **LEETCHI_PAYER_HELPER** is missing.')
elif isinstance(get_payer, six.string_types):
    get_payer = load_class(get_payer)


def _get_wallet(resource):
    from .model import Wallet

    return Wallet.objects.get_for_model(resource).wallet


get_wallet = getattr(settings, 'LEETCHI_WALLET_HELPER', _get_wallet)

if not get_wallet:
    raise ImproperlyConfigured('The required setting **LEETCHI_WALLET_HELPER** is missing.')
elif isinstance(get_wallet, six.string_types):
    get_wallet = load_class(get_wallet)



# -*- coding: utf-8 -*-
import django

from django.utils import translation
from django.conf import settings
from django.core import exceptions
from django.utils.importlib import import_module

CLASS_PATH_ERROR = 'djleetchi is unable to interpret settings value for %s. '\
                   '%s should be in the form of a tupple: '\
                   '(\'path.to.models.Class\', \'app_label\').'


def get_current_lang():
    try:
        lang = translation.get_language()[:2]
        if lang not in [l[0] for l in settings.LANGUAGES]:
            raise
    except:
        lang = settings.LANGUAGE_CODE
    return lang


def load_class(class_path, setting_name=None):
    """
    Loads a class given a class_path. The setting value may be a string or a
    tuple.

    The setting_name parameter is only there for pretty error output, and
    therefore is optional
    """
    if not isinstance(class_path, basestring):
        try:
            class_path, app_label = class_path
        except:
            if setting_name:
                raise exceptions.ImproperlyConfigured(CLASS_PATH_ERROR % (
                    setting_name, setting_name))
            else:
                raise exceptions.ImproperlyConfigured(CLASS_PATH_ERROR % (
                    'this setting', 'It'))

    try:
        class_module, class_name = class_path.rsplit('.', 1)
    except ValueError:
        if setting_name:
            txt = '%s isn\'t a valid module. Check your %s setting' % (
                class_path, setting_name)
        else:
            txt = '%s isn\'t a valid module.' % class_path
        raise exceptions.ImproperlyConfigured(txt)

    try:
        mod = import_module(class_module)
    except ImportError, e:
        if setting_name:
            txt = 'Error importing backend %s: "%s". Check your %s setting' % (
                class_module, e, setting_name)
        else:
            txt = 'Error importing backend %s: "%s".' % (class_module, e)

        raise exceptions.ImproperlyConfigured(txt)

    try:
        clazz = getattr(mod, class_name)
    except AttributeError:
        if setting_name:
            txt = ('Backend module "%s" does not define a "%s" class. Check'
                   ' your %s setting' % (class_module, class_name,
                                         setting_name))
        else:
            txt = 'Backend module "%s" does not define a "%s" class.' % (
                class_module, class_name)
        raise exceptions.ImproperlyConfigured(txt)
    return clazz

if django.VERSION >= (1, 5):
    update_fields = lambda instance, fields: instance.save(update_fields=fields)
else:
    update_fields = lambda instance, fields: instance.save()



from optparse import make_option

from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('--limit',
                    action='store',
                    type='int',
                    dest='limit',
                    default=100,
                    help='Limit resources to sync'),
        make_option('--name',
                    action='store',
                    type='string',
                    dest='name',
                    default=None,
                    help='Name of resource you want to sync'),
    )

    can_import_settings = True

    @property
    def resources(self):
        from djleetchi.models import Contribution, Withdrawal, Refund

        return {
            'refund': Refund,
            'withdrawal': Withdrawal,
            'contribution': Contribution
        }

    def handle(self, *args, **options):
        from djleetchi.tasks import sync_status

        from celery import group

        resources = self.resources

        name = options.get('name', None)

        if name and name not in resources:
            raise CommandError('Resource name %s does not exist or not available to sync' % name)

        limit = options.get('limit')

        for resource_name, klass in resources.iteritems():
            if name and resource_name != name:
                continue

            params = {
                'is_completed': False,
                '%s__isnull' % klass.Api.resource_field: False
            }

            resource_ids = klass.objects.filter(**params).values_list('id', flat=True)[:limit]

            tasks = [sync_status.subtask(args=(klass, resource_id, )) for resource_id in resource_ids]

            job = group(tasks)

            job.apply_async()

            print 'Syncing %s %s' % (limit, resource_name)
